# Copyright 2016-2018 Dirk Thomas
# Licensed under the Apache License, Version 2.0

from colcon_core.entry_point import get_all_entry_points
from colcon_core.entry_point import load_entry_point
from colcon_core.plugin_system import get_first_line_doc
from colcon_core.plugin_system import satisfies_version
from colcon_core.verb import VerbExtensionPoint


class ExtensionsVerb(VerbExtensionPoint):
    """List extensions."""

    def __init__(self):  # noqa: D107
        super().__init__()
        satisfies_version(VerbExtensionPoint.EXTENSION_POINT_VERSION, '^1.0')

    def add_arguments(self, *, parser):  # noqa: D102
        all_entry_points = get_all_entry_points()
        parser.add_argument(
            'group_name',
            nargs='?',
            choices=sorted(list(all_entry_points.keys())),
            metavar='GROUP_NAME',
            help='Only show the extensions in a specific group')
        parser.add_argument(
            '--all', '-a',
            action='store_true',
            default=False,
            help='Also show extensions which failed to load or are '
                 'incompatible. (prefixed with `- `)')
        parser.add_argument(
            '--verbose', '-v',
            action='store_true',
            default=False,
            help='Show more information for each extension')

    def main(self, *, context):  # noqa: D102
        all_entry_points = get_all_entry_points()
        for group_name in sorted(all_entry_points.keys()):
            if context.args.group_name is not None:
                if group_name != context.args.group_name:
                    continue
            print(group_name)
            group = all_entry_points[group_name]
            for entry_point_name in sorted(group.keys()):
                (dist, entry_point) = group[entry_point_name]
                self._print_entry_point(context.args, dist, entry_point)

    def _print_entry_point(self, args, dist, entry_point):
        exception = None
        try:
            extension = load_entry_point(entry_point)
        except Exception as e:
            # catch exceptions raised when loading entry point
            if not args.all:
                # skip entry points which failed to load
                return
            exception = e
            extension = None
        else:
            try:
                extension()
            except Exception as e:
                # catch exceptions raised in extension constructor
                if not args.all:
                    # skip extensions which failed to be instantiated
                    return
                exception = e

        prefix = ' ' if exception is None else '-'
        print(prefix, entry_point.name + ':', get_first_line_doc(extension))

        if args.verbose:
            print(prefix, ' ', 'module_name:', entry_point.module_name)
            if entry_point.attrs:
                print(prefix, ' ', 'attributes:', '.'.join(entry_point.attrs))
            print(prefix, ' ', 'distribution:', repr(dist))
            try:
                print(prefix, ' ', 'priority:', extension.PRIORITY)
            except AttributeError:
                pass

        if exception:
            print(prefix, ' ', 'reason:', str(exception))



# Copyright 2016-2018 Dirk Thomas
# Licensed under the Apache License, Version 2.0

from colcon_core.entry_point import EXTENSION_POINT_GROUP_NAME
from colcon_core.entry_point import get_entry_points
from colcon_core.entry_point import load_entry_point
from colcon_core.plugin_system import get_first_line_doc
from colcon_core.plugin_system import satisfies_version
from colcon_core.verb import VerbExtensionPoint


class ExtensionPointsVerb(VerbExtensionPoint):
    """List extension points."""

    def __init__(self):  # noqa: D107
        super().__init__()
        satisfies_version(VerbExtensionPoint.EXTENSION_POINT_VERSION, '^1.0')

    def add_arguments(self, *, parser):  # noqa: D102
        parser.add_argument(
            '--all', '-a',
            action='store_true',
            default=False,
            help='Also show extension points which failed to be imported. '
                 '(prefixed with `- `)')
        parser.add_argument(
            '--verbose', '-v',
            action='store_true',
            default=False,
            help='Show more information for each extension point')

    def main(self, *, context):  # noqa: D102
        colcon_extension_points = get_entry_points(EXTENSION_POINT_GROUP_NAME)
        for name in sorted(colcon_extension_points.keys()):
            # skip "private" extension points
            if name.startswith('_'):
                continue
            self._print_extension_point(
                context.args, name, colcon_extension_points[name])

    def _print_extension_point(self, args, name, entry_point):
        exception = None
        try:
            extension_point = load_entry_point(entry_point)
        except Exception as e:
            # catch exceptions raised when loading entry point
            if not args.all:
                # skip entry points which failed to load
                return
            exception = e
            extension_point = None

        prefix = '' if exception is None else '- '
        print(prefix + name + ':', get_first_line_doc(extension_point))

        if args.verbose:
            print(prefix, ' ', 'module_name:', entry_point.module_name)
            if entry_point.attrs:
                print(prefix, ' ', 'attributes:', '.'.join(entry_point.attrs))
            if hasattr(extension_point, 'EXTENSION_POINT_VERSION'):
                print(
                    prefix, ' ', 'version:',
                    extension_point.EXTENSION_POINT_VERSION)

        if exception:
            print(prefix, ' ', 'reason:', str(exception))



# Copyright 2016-2018 Dirk Thomas
# Licensed under the Apache License, Version 2.0

import sys
from xmlrpc.client import ServerProxy

from colcon_core.entry_point import EXTENSION_POINT_GROUP_NAME
from colcon_core.entry_point import get_all_entry_points
from colcon_core.entry_point import get_entry_points
from colcon_core.plugin_system import satisfies_version
from colcon_core.verb import VerbExtensionPoint
from pkg_resources import parse_version


class VersionCheckVerb(VerbExtensionPoint):
    """Compare local package versions with PyPI."""

    def __init__(self):  # noqa: D107
        super().__init__()
        satisfies_version(VerbExtensionPoint.EXTENSION_POINT_VERSION, '^1.0')

    def main(self, *, context):  # noqa: D102
        distributions = set()
        all_entry_points = get_all_entry_points()
        for group in all_entry_points.values():
            for dist, _ in group.values():
                distributions.add(dist)

        # also consider extension points which don't have any extensions
        colcon_extension_points = get_entry_points(EXTENSION_POINT_GROUP_NAME)
        for entry_point in colcon_extension_points.values():
            distributions.add(entry_point.dist)

        pypi = ServerProxy('https://pypi.python.org/pypi')
        for dist in sorted(distributions, key=lambda d: d.project_name):
            versions = pypi.package_releases(dist.project_name)
            if not versions:
                print(
                    '{dist.project_name}: could not find package on PyPI'
                    .format_map(locals()), file=sys.stderr)
                continue

            latest_version = versions[0]
            if parse_version(latest_version) == parse_version(dist.version):
                print(
                    '{dist.project_name} {dist.version}: up-to-date'
                    .format_map(locals()))
                continue
            if parse_version(dist.version) < parse_version(latest_version):
                print(
                    '{dist.project_name} {dist.version}: newer version '
                    'available ({latest_version})'
                    .format_map(locals()))
                continue
            print(
                '{dist.project_name} {dist.version}: local version is newer '
                'than latest release ({latest_version})'
                .format_map(locals()))
            continue
